/*	ljm.h -- ljsdcdev
	.ljm stuff
	CrabClaw Engine
	Copright (C) 2003, 2004 Lawrence Sebald

	This program is free software; you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation; either version 2 of the License, or
	(at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program; if not, write to the Free Software
	Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#include <GL/gl.h>
#include "cctypes.h"

#ifndef __LJM_H
#define __LJM_H

#ifdef __cplusplus
extern "C" {
#endif

typedef struct	{
	uint32 user1;
	uint16 user2, user3;
	uint8 user4, user5, user6;
} ljm_tile_t;

typedef struct	{
	uint32 evx, evy;
	uint8 evtype;
	uint32 evd1, evd2, evd3;
} ljm_event_t;

typedef struct	{
	uint32 count;
	uint32 * seq;
	uint8 delay;
	uint64 sttime;
	uint32 current;
} ljm_anim_t;

typedef struct	{
	GLuint * tiles_txr;
	uint32 ** tile_map;
	uint8 dim;
	uint32 map_x;
	uint32 map_y;
	uint32 xoff;
	uint32 yoff;
	uint32 version;
	uint32 tcount;
	uint32 ecount;
	uint32 acount;
	ljm_tile_t * tile;
	ljm_event_t * event;
	ljm_anim_t * anim;
} ljm_map_t;

/* Functions*/
ljm_map_t * ljm_load(const char * fn);
int ljm_free(ljm_map_t * map);
ljm_tile_t * ljm_get_data(ljm_map_t * map, uint32 x, uint32 y);
ljm_tile_t * ljm_get_blk(ljm_map_t * map, uint32 blknum);
ljm_tile_t * ljm_get_data_pixels(ljm_map_t * map, uint32 x, uint32 y);
int ljm_set_xoffset(ljm_map_t * map, uint32 off);
int ljm_set_yoffset(ljm_map_t * map, uint32 off);
int ljm_map_scroll(ljm_map_t * map, uint32 xpos, uint32 ypos);
int ljm_map_center(ljm_map_t * map, uint32 x, uint32 y, uint32 scx, uint32 scy);
void ljm_draw_bg(ljm_map_t * map);
void ljm_inc_anims(ljm_map_t * map);
ljm_event_t * ljm_scan_events(ljm_map_t * map, uint32 x, uint32 y);
ljm_event_t * ljm_scan_events_type(ljm_map_t * map, uint32 x, uint32 y, uint8 type);

#ifdef __cplusplus
}
#endif

#endif /* !__LJM_H */
