/*	ccfont.c -- ljsdcdev
	libcrabclaw font functions
	libcrabclaw
	Copyright (C) 2004 Lawrence Sebald	*/

/*	This font stuff draws a lot from the NeHe
	FreeType font tutorial which is available
	at http://nehe.gamedev.net/		*/

#include "cctypes.h"
#include "ccfont.h"
#include <stdio.h>
#include <string.h>
#include <math.h>
#include <GL/gl.h>
#include <ft2build.h>
#include FT_FREETYPE_H

static FT_Library lib;

static inline int next_p2(int a)	{
	int r = 1;
	while(r < a)
		r <<= 1;

	return r;
}

static inline void __copy_bitmap(FT_Face face, char * data, int w, int h)	{
	int i, j;
	FT_Bitmap bmp = face->glyph->bitmap;

	memset(data, 0, w * h * 2);

	for(i = 0; i < bmp.rows; i++)	{
		for(j = 0; j < bmp.width; j++)	{
			data[2 * (j + i * w)] =
			data[2 * (j + i * w) + 1] =
			bmp.buffer[j + bmp.width * i];
		}
	}
}

void cc_font_init()	{
	if (FT_Init_FreeType(&lib))
		fprintf(stderr, "cc_font_init: Could not initialize FreeType!\n");
}

void cc_font_deinit()	{
}

int cc_font_load(const char * fn, cc_font_t * font, int type, float he)	{
	FT_Face face;
	int error;
	int i;
	char * data;
	int w, h;

	error = FT_New_Face(lib, fn, 0, &face);
	if(error == FT_Err_Unknown_File_Format)	{
		fprintf(stderr, "cc_font_load: Unknown file format in file: %s\n", fn);
		return error;
	}
	else if(error != 0)	{
		fprintf(stderr, "cc_load_font: Unable to load font %s\n", fn);
		return error;
	}

	FT_Set_Char_Size(face, he * 64, he * 64, 96, 96);

	font->h = he;

	font->tex = (GLuint *) malloc(sizeof(GLuint) * 224);
	font->gdat = (cc_glyph_t *) malloc(sizeof(cc_glyph_t) * 224);
	glGenTextures(224, font->tex);
	for(i = 32; i < 256; i++)	{
		error = FT_Load_Char(face, i, FT_LOAD_RENDER);
		if(error)	{
			fprintf(stderr, "cc_load_font: Unable to load character %c\n", i);
			continue;
		}

		w = next_p2(face->glyph->bitmap.width);
		h = next_p2(face->glyph->bitmap.rows);
		font->gdat[i - 32].w = face->glyph->bitmap.width;
		font->gdat[i - 32].h = face->glyph->bitmap.rows;
		font->gdat[i - 32].u = (float) face->glyph->bitmap.width / w;
		font->gdat[i - 32].v = (float) face->glyph->bitmap.rows / h;
		font->gdat[i - 32].below = (float) (face->glyph->metrics.height -
		                                   face->glyph->metrics.horiBearingY) / 64;
		font->gdat[i - 32].adv = (float) face->glyph->metrics.horiAdvance / 64;
		data = (char *) malloc(w * h * 2);

		__copy_bitmap(face, data, w, h);

		glBindTexture(GL_TEXTURE_2D, font->tex[i - 32]);
		glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR);
		glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR);
		glTexImage2D(GL_TEXTURE_2D, 0, GL_RGBA, w, h, 0, GL_LUMINANCE_ALPHA,
		GL_UNSIGNED_BYTE, data);
		free(data);
	}

	FT_Done_Face(face);

	return 0;
}

void cc_font_destroy(cc_font_t * font)	{
	glDeleteTextures(224, font->tex);
	free(font->gdat);
}

void cc_print(cc_font_t * font, float x, float y, float he, const char * str)	{
	int i, num;
	float w, h, ox, u, v, factor, off, below, adv, hi;

	y -= (he - 2);
	ox = x;
	factor = he / font->h;
	hi = he - floor(he / 3);


	glEnable(GL_TEXTURE_2D);
	for(i = 0; str[i] != '\0'; i++)	{
		if(str[i] == '\n')	{
			y += he + 2;
			x = ox;
		}
		else if(str[i] == ' ')	{
			x += .5f * he;
		}
		else	{
			num = str[i] - 32;
			below = font->gdat[num].below;
			w = font->gdat[num].w * factor;
			h = font->gdat[num].h * factor - below;
			u = font->gdat[num].u;
			v = font->gdat[num].v;
			adv = font->gdat[num].adv * factor;
			off = hi - h;

			glBindTexture(GL_TEXTURE_2D, font->tex[str[i] - 32]);
			glBegin(GL_QUADS);
				glTexCoord2f(0.0f, 0.0f);
				glVertex3f(x, y + off, 0.1f);
				glTexCoord2f(u, 0.0f);
				glVertex3f(x + w, y + off, 0.1f);
				glTexCoord2f(u, v);
				glVertex3f(x + w, y + hi + below, 0.1f);
				glTexCoord2f(0.0f, v);
				glVertex3f(x, y + hi + below, 0.1f);
			glEnd();
			x += adv;
		}
	}
	glDisable(GL_TEXTURE_2D);
}

void cc_printf(cc_font_t * font, float x, float y, float he, const char * fmt, ...)	{
	char str[1024];
	va_list ap;

	if(fmt == NULL)	{
		return;
	}

	va_start(ap, fmt);
	vsprintf(str, fmt, ap);
	va_end(ap);

	cc_print(font, x, y, he, str);
}

void cc_fcolored_printf(cc_font_t * font, float x, float y, float he,
                        float a, float r, float g, float b, const char * fmt, ...)	{
	char str[1024];
	va_list ap;

	if(fmt == NULL)	{
		return;
	}

	va_start(ap, fmt);
	vsprintf(str, fmt, ap);
	va_end(ap);

	glColor4f(r, g, b, a);
	cc_print(font, x, y, he, str);
	glColor4f(1.0f, 1.0f, 1.0f, 1.0f);
}

void cc_icolored_printf(cc_font_t * font, float x, float y, float he,
                        uint8 a, uint8 r, uint8 g, uint8 b, const char * fmt, ...)	{
	char str[1024];
	va_list ap;

	if(fmt == NULL)	{
		return;
	}

	va_start(ap, fmt);
	vsprintf(str, fmt, ap);
	va_end(ap);

	glColor4f(r / 255.0f, g / 255.0f, b / 255.0f, a / 255.0f);
	cc_print(font, x, y, he, str);
	glColor4f(1.0f, 1.0f, 1.0f, 1.0f);
}

void cc_centered_print(cc_font_t * font, float x1, float x2, float y, float h, const char * str)	{
	float factor, posx, w;
	int i, off;
	char string[1024];

	factor = h / font->h;
	w = 0;
	off = 0;

	for(i = 0; str[i]; i++)	{
		if(str[i] == '\n')	{
			strncpy(string, &str[off], i - off);
			posx = ((x2 - x1 - w) / 2) + x1;
			cc_print(font, posx, y, h, string);
			memset(string, 0, 1024);
			y += h;
			w = 0;
			off = i;
			continue;
		}
		w += (font->gdat[str[i] - 32].adv * factor);
	}

	posx = ((x2 - x1 - w) / 2) + x1;
	strncpy(string, &str[off], i - off);
	cc_print(font, posx, y, h, string);
}

void cc_centered_printf(cc_font_t * font, float x1, float x2, float y, float he, const char * fmt, ...)	{
	char str[1024];
	va_list ap;

	if(fmt == NULL)	{
		return;
	}

	va_start(ap, fmt);
	vsprintf(str, fmt, ap);
	va_end(ap);

	cc_centered_print(font, x1, x2, y, he, str);
}
