/*	ccvideo.c -- ljsdcdev
	libcrabclaw video functions
	libcrabclaw
	Copyright (C) 2004 Lawrence Sebald	*/

#include "cctypes.h"
#include "ccvideo.h"
#include <GL/glx.h>
#include <GL/gl.h>
#include <X11/Xlib.h>
#include <string.h>
#include <malloc.h>

Display * disp;
static GLXContext cxt;
static Window win;
static char * title;
static int w = 640, h = 480;
static int x = 0, y = 0;
static int depth;
int screen;

void cc_video_init()	{
	int glxAttrib[] = {GLX_RGBA, GLX_RED_SIZE, 4, GLX_GREEN_SIZE, 4, GLX_BLUE_SIZE, 4,
	                   GLX_DOUBLEBUFFER, GLX_DEPTH_SIZE, 16, None};
	int glxAttrib2[] = {GLX_RGBA, GLX_RED_SIZE, 4, GLX_GREEN_SIZE, 4, GLX_BLUE_SIZE, 4,
	                    GLX_DEPTH_SIZE, 16, None};
	Window root;
	XVisualInfo * vi;
	XSetWindowAttributes attr;
	Window winDummy;
	unsigned int borderDummy;
	Atom wmDelete;
	
	disp = XOpenDisplay(0);
	screen = DefaultScreen(disp);
	root = RootWindow(disp, screen);
	vi = glXChooseVisual(disp, screen, glxAttrib);
	if(!vi)	{
		vi = glXChooseVisual(disp, screen, glxAttrib2);
	}

	cxt = glXCreateContext(disp, vi, 0, True);

	attr.colormap = XCreateColormap(disp, root, vi->visual, AllocNone);
	attr.border_pixel = 0;
	attr.event_mask = ExposureMask | StructureNotifyMask | KeyPressMask;
	win = XCreateWindow(disp, root, 0, 0, w, h, 0, vi->depth, InputOutput,
	                    vi->visual, CWBorderPixel | CWColormap | CWEventMask, &attr);

	wmDelete = XInternAtom(disp, "WM_DELETE_WINDOW", True);
	XSetWMProtocols(disp, win, &wmDelete, 1);
	XSetStandardProperties(disp, win, title, title, None, NULL, 0, NULL);
	XMapRaised(disp, win);
	glXMakeCurrent(disp, win, cxt);
	XGetGeometry(disp, win, &winDummy, &x, &y, &w, &h, &borderDummy, &depth);

	glShadeModel(GL_SMOOTH);
	glClearColor(0.0f, 0.0f, 0.0f, 0.0f);
	glClearDepth(1.0f);
	glDisable(GL_DEPTH_TEST);
	glDepthFunc(GL_LEQUAL);
	glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
	glEnable(GL_BLEND);
	glHint(GL_PERSPECTIVE_CORRECTION_HINT, GL_NICEST);
	glViewport(0, 0, w, h);
	glMatrixMode(GL_PROJECTION);
	glLoadIdentity();
	glOrtho(0.0f, w, h, 0.0f, -1.0f, 1.0f);
	glMatrixMode(GL_MODELVIEW);
	glLoadIdentity();
	glFlush();
}

void cc_scene_begin()	{
	glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
	glLoadIdentity();
}

void cc_scene_finish()	{
	glXSwapBuffers(disp, win);
}

void cc_draw_tex(cc_tex_t tex, float x, float y, float w, float h)	{
	glEnable(GL_TEXTURE_2D);
	glBindTexture(GL_TEXTURE_2D, (GLuint) tex);
	glBegin(GL_QUADS);
		glTexCoord2f(0.0f, 0.0f);
		glVertex3f(x, y, 0.0f);
		glTexCoord2f(1.0f, 0.0f);
		glVertex3f(x + w, y, 0.0f);
		glTexCoord2f(1.0f, 1.0f);
		glVertex3f(x + w, y + h, 0.0f);
		glTexCoord2f(0.0f, 1.0f);
		glVertex3f(x, y + h, 0.0f);
	glEnd();
	glDisable(GL_TEXTURE_2D);
}

void cc_video_deinit()	{
	if(cxt)	{
		glXMakeCurrent(disp, None, NULL);
		glXDestroyContext(disp, cxt);
	}
	XCloseDisplay(disp);
}

void cc_set_title(const char * name)	{
	title = (char *) malloc(strlen(name) + 1);
	strcpy(title, name);
}

void cc_video_get_params(int * wi, int * he, int * dep)	{
	*wi = w;
	*he = h;
	*dep = depth;
}

void cc_destroy_tex(cc_tex_t * tex)	{
	glDeleteTextures(1, tex);
}
