/*	ljm.c -- ljsdcdev
	.ljm stuff
	libcrabclaw
	Copright (C) 2003, 2004 Lawrence Sebald

	This program is free software; you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation; either version 2 of the License, or
	(at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program; if not, write to the Free Software
	Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#include <GL/gl.h>
#include <stdio.h>
#include <malloc.h>
#include <string.h>
#include "cctimer.h"
#include "cctypes.h"
#include "ljm.h"

ljm_map_t * ljm_load(const char * fn)	{
	unsigned long i;
	FILE * fp;
	char * tmp;
	uint64 sttime;
	char sig[4];
	ljm_map_t * map = (ljm_map_t *) malloc(sizeof(ljm_map_t));

	fp = fopen(fn, "rb");
	if(fp == NULL)	{
		fprintf(stderr, "ljm_load: Cannot open file: %s\n", fn);
		return NULL;
	}

	fread(sig, 3, 1, fp);
	if(strncmp(sig, "LJM", 3))	{
		fprintf(stderr, "ljm_load: File %s is not a valid ljm map.\n", fn);
		return NULL;
	}

	fread(&map->version, sizeof(uint32), 1, fp);
	fread(&map->dim, sizeof(uint8), 1, fp);
	fread(&map->tcount, sizeof(uint32), 1, fp);
	fread(&map->map_x, sizeof(uint32), 1, fp);
	fread(&map->map_y, sizeof(uint32), 1, fp);

	if(map->version != 1)	{
		fread(&map->ecount, sizeof(uint32), 1, fp);
		fread(&map->acount, sizeof(uint32), 1, fp);
	}

	map->tile = (ljm_tile_t *) malloc(sizeof(ljm_tile_t) * map->tcount);
	fread(map->tile, sizeof(ljm_tile_t), map->tcount, fp);

	if(map->version != 1)	{
		map->event = (ljm_event_t *) malloc(sizeof(ljm_event_t) * map->ecount);
		map->anim = (ljm_anim_t *) malloc(sizeof(ljm_anim_t) * map->acount);
		fread(map->event, sizeof(ljm_event_t), map->ecount, fp);
		for(i = 0; i < map->acount; i++)	{
			fread(&map->anim[i].count, sizeof(uint32), 1, fp);
			map->anim[i].seq = (uint32 *) malloc(sizeof(uint32) * map->anim[i].count);
			fread(map->anim[i].seq, sizeof(uint32), map->anim[i].count, fp);
			fread(&map->anim[i].delay, sizeof(uint8), 1, fp);
		}
	}

	map->tiles_txr = (GLuint *) malloc(sizeof(GLuint) * map->tcount);
	glGenTextures(map->tcount, &map->tiles_txr[0]);

	tmp = (char *) malloc(map->dim * map->dim * 2);

	for(i = 0; i < map->tcount; i++)	{
		fread(tmp, map->dim * map->dim * 2, 1, fp);
		glBindTexture(GL_TEXTURE_2D, map->tiles_txr[i]);
		glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR);
		glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR);
		glTexImage2D(GL_TEXTURE_2D, 0, 3, map->dim, map->dim, 0, GL_RGB, GL_UNSIGNED_SHORT_5_6_5, tmp);
	}

	free(tmp);

	map->tile_map = (uint32 **) malloc(sizeof(uint32 *) * map->map_y);
	for(i = 0; i < map->map_y; i++)	{
		map->tile_map[i] = (uint32 *) malloc(sizeof(uint32) * map->map_x);
		fread(map->tile_map[i], sizeof(uint32), map->map_x, fp);
	}

	fclose(fp);

	if(map->version != 1)	{
		sttime = cc_timer_get_ticks();
		for(i = 0; i < map->acount; i++)	{
			map->anim[i].sttime = sttime;
			map->anim[i].current = 0;
		}
	}

	map->xoff = map->yoff = 0;

	return map;
}

int ljm_free(ljm_map_t * map)	{
	uint32 i;

	glDeleteTextures(map->tcount, map->tiles_txr);

	free(map->tile);

	for(i = 0; i < map->map_y; i++)	{
		free(map->tile_map[i]);
	}

	free(map->tile_map);

	if(map->version != 1)	{
		for(i = 0; i < map->acount; i++)	{
			free(map->anim[i].seq);
		}
		free(map->anim);
		free(map->event);
	}

	free(map);

	map = NULL;

	return 0;
}

ljm_tile_t * ljm_get_data_blk(ljm_map_t * map, uint32 blk)	{
	if(blk < 2147483649UL)	{
		return &map->tile[blk];
	}
	else	{
		return &map->tile[map->anim[blk - 2147483649UL].seq[0]];
	}
}

ljm_tile_t * ljm_get_data_pixels(ljm_map_t * map, uint32 x, uint32 y)	{
	return ljm_get_data_blk(map, map->tile_map[y / map->dim][x / map->dim]);
}

ljm_tile_t * ljm_get_data(ljm_map_t * map, uint32 x, uint32 y)	{
	return ljm_get_data_blk(map, map->tile_map[y][x]);
}

int ljm_set_xoffset(ljm_map_t * map, uint32 off)	{
	if(off > map->map_x)	{
		fprintf(stderr, "ljm_set_xoffset: offset %lu is larger than the map's x size", off);
		return -1;
	}

	map->xoff = off;

	return 0;
}

int ljm_set_yoffset(ljm_map_t * map, uint32 off)	{
	if(off > map->map_y)	{
		fprintf(stderr, "ljm_set_yoffset: offset %lu is larger than the map's y size", off);
		return -1;
	}

	map->yoff = off;

	return 0;
}

int ljm_map_scroll(ljm_map_t * map, uint32 x, uint32 y)	{
	if(ljm_set_xoffset(map, x) == 0 && ljm_set_yoffset(map, y))	{
		return 0;
	}

	return -1;
}

int ljm_map_center(ljm_map_t * map, uint32 x, uint32 y, uint32 scx, uint32 scy)	{
	return ljm_map_scroll(map, x - ((scx / 2) / map->dim), y - ((scy / 2) / map->dim));
}

void ljm_inc_anims(ljm_map_t * map)	{
	uint64 timer = cc_timer_get_ticks();
	uint32 i;

	for(i = 0; i < map->acount; i++)	{
		if(map->anim[i].sttime + (map->anim[i].delay * 100) <= timer)	{
			map->anim[i].sttime = timer;
			if(map->anim[i].current + 1 == map->anim[i].count)	{
				map->anim[i].current = 0;
			}
			else	{
				map->anim[i].current++;
			}
		}
	}
}

ljm_event_t * ljm_scan_events(ljm_map_t * map, uint32 x, uint32 y)	{
	uint32 i;
	ljm_event_t * eventfound = NULL;

	for(i = 0; i < map->ecount; i++)	{
		if(map->event[i].evx == x && map->event[i].evy == y)	{
			eventfound = &map->event[i];
			break;
		}
	}

	return eventfound;
}

ljm_event_t * ljm_scan_events_type(ljm_map_t * map, uint32 x, uint32 y, uint8 type)	{
	ljm_event_t * eventfound = ljm_scan_events(map, x, y);

	if(eventfound != NULL && eventfound->evtype == type)	{
		return eventfound;
	}

	return NULL;
}

void ljm_draw_bg(ljm_map_t * map)	{
	uint32 i, j, animnum, animcur;
	float x, y;
	glEnable(GL_TEXTURE_2D);

	for(i = 0; i + map->yoff < map->map_y; i++)	{
		y = i * map->dim;
		for(j = 0; j + map->xoff < map->map_x ; j++)	{
			x = j * map->dim;
			if((i + map->yoff) * map->dim < 480 && (j + map->xoff) * map->dim < 640)	{
				if(map->tile_map[i + map->yoff][j + map->xoff] < 2147483649UL)	{
					glBindTexture(GL_TEXTURE_2D, map->tiles_txr[map->tile_map[i + map->yoff][j + map->xoff]]);
					glBegin(GL_QUADS);
						glTexCoord2f(0.0f, 0.0f);
						glVertex3f(x, y, 0.0f);
						glTexCoord2f(1.0f, 0.0f);
						glVertex3f(x + map->dim, y, 0.0f);
						glTexCoord2f(1.0f, 1.0f);
						glVertex3f(x + map->dim, y + map->dim, 0.0f);
						glTexCoord2f(0.0f, 1.0f);
						glVertex3f(x, y + map->dim, 0.0f);
					glEnd();
				}
				else	{
					animnum = map->tile_map[i + map->yoff][j + map->xoff] - 2147483649UL;
					animcur = map->anim[animnum].current;
					glBindTexture(GL_TEXTURE_2D, map->tiles_txr[map->anim[animnum].seq[animcur]]);
					glBegin(GL_QUADS);
						glTexCoord2f(0.0f, 0.0f);
						glVertex3f(x, y, 0.0f);
						glTexCoord2f(1.0f, 0.0f);
						glVertex3f(x + map->dim, y, 0.0f);
						glTexCoord2f(1.0f, 1.0f);
						glVertex3f(x + map->dim, y + map->dim, 0.0f);
						glTexCoord2f(0.0f, 1.0f);
						glVertex3f(x, y + map->dim, 0.0f);
					glEnd();
				}
			}
		}
	}
}
