/*	readpng.c -- ljsdcdev
	libcrabclaw PNG reader
	libcrabclaw
	Copyright (C) 2004 Lawrence Sebald	*/

#include "cctypes.h"
#include "ccvideo.h"
#include <png.h>
#include <stdio.h>
#include <malloc.h>
#include <math.h>
#include <GL/gl.h>
#include <GL/glu.h>

int cc_load_png(const char * fn, cc_tex_t * num, uint32 flags)	{
	FILE * fp;
	char hdr[9];
	png_structp png_ptr;
	png_infop info_ptr;
	png_infop end_info;
	png_bytepp rowpointers;
	png_uint_32 height, width, rowbytes;
	int bit_depth, color_type, channels, type, type2;
	int i, j;
	char * data;

	fp = fopen(fn, "rb");
	if(!fp)	{
		fprintf(stderr, "cc_load_png: Error opening %s\n", fn);
		fclose(fp);
		return -1;
	}

	fread(hdr, 1, 8, fp);
	if(png_sig_cmp(hdr, 0, 8))	{
		fprintf(stderr, "cc_load_png: %s is not a PNG\n", fn);
		fclose(fp);
		return -1;
	}

	png_ptr = png_create_read_struct(PNG_LIBPNG_VER_STRING, NULL, NULL, NULL);
	if(!png_ptr)	{
		fprintf(stderr, "cc_load_png: Couldn't create PNG structure\n");
		fclose(fp);
		return -1;
	}

	info_ptr = png_create_info_struct(png_ptr);
	if(!info_ptr)	{
		png_destroy_read_struct(&png_ptr, (png_infopp)NULL, (png_infopp)NULL);
		fprintf(stderr, "cc_load_png: Couldn't create PNG info structure\n");
		fclose(fp);
		return -1;
	}

	end_info = png_create_info_struct(png_ptr);
	if (!end_info)	{
		png_destroy_read_struct(&png_ptr, &info_ptr, (png_infopp)NULL);
		fprintf(stderr, "cc_load_png: Couldn't create PNG info structure\n");
		fclose(fp);
		return -1;
	}

	if (setjmp(png_jmpbuf(png_ptr)))	{
		png_destroy_read_struct(&png_ptr, &info_ptr, &end_info);
		fprintf(stderr, "cc_load_png: Couldn't set PNG jmpbuf\n");
		fclose(fp);
		return -1;
	}

	png_init_io(png_ptr, fp);
	png_set_sig_bytes(png_ptr, 8);
	png_read_info(png_ptr, info_ptr);

	png_get_IHDR(png_ptr, info_ptr, &width, &height, &bit_depth, &color_type, NULL, NULL, NULL);

	if(color_type == PNG_COLOR_TYPE_PALETTE)
		png_set_palette_to_rgb(png_ptr);
	if(color_type == PNG_COLOR_TYPE_GRAY && bit_depth < 8)
		png_set_gray_1_2_4_to_8(png_ptr);
	if(png_get_valid(png_ptr, info_ptr, PNG_INFO_tRNS))
		png_set_tRNS_to_alpha(png_ptr);
	if(bit_depth == 16)
		png_set_strip_16(png_ptr);
	if(color_type == PNG_COLOR_TYPE_GRAY || color_type == PNG_COLOR_TYPE_GRAY_ALPHA)
		png_set_gray_to_rgb(png_ptr);
	if(bit_depth < 8)
		png_set_packing(png_ptr);

	png_read_update_info(png_ptr, info_ptr);

	rowbytes = png_get_rowbytes(png_ptr, info_ptr);
	channels = (int) png_get_channels(png_ptr, info_ptr);

	data = (char *) malloc(rowbytes * height);
	if(data == NULL)	{
		png_destroy_read_struct(&png_ptr, &info_ptr, &end_info);
		fprintf(stderr, "cc_load_png: Couldn't allocate memory for image\n");
		fclose(fp);
		return -1;
	}

	rowpointers = (png_bytepp )malloc(height * sizeof(png_bytep));
	if(rowpointers == NULL)	{
		png_destroy_read_struct(&png_ptr, &info_ptr, &end_info);
		fprintf(stderr, "cc_load_png: Couldn't allocate memory for rowpointers\n");
		free(data);
		fclose(fp);
		return -1;
	}

	for(i = 0; i < height; i++)	{
		rowpointers[i] = data + (i * rowbytes);
	}

	png_read_image(png_ptr, rowpointers);

	png_read_end(png_ptr, end_info);
	png_destroy_read_struct(&png_ptr, &info_ptr, &end_info);

	/* Invert the texture over the X axis, if requested */
	if(flags & CC_INVERT_X)	{
		for(i = 0; i < height; i++)	{
			if(rowbytes / width == 1)	{
				unsigned char * image_data = (unsigned char *) rowpointers[i];
				unsigned char buf;
				for(j = 0; j < width / 2; j++)	{
					buf = image_data[j];
					image_data[j] = image_data[width - j];
					image_data[width - j] = buf;
				}
			}
			else if(rowbytes / width == 2)	{
				unsigned short * image_data = (unsigned short *) rowpointers[i];
				unsigned short buf;
				for(j = 0; j < width / 2; j++)	{
					buf = image_data[j];
					image_data[j] = image_data[width - j];
					image_data[width - j] = buf;
				}
			}
			else if(rowbytes / width == 3)	{
				unsigned char * image_data = (unsigned char *) rowpointers[i];
				unsigned char buf[3];
				for(j = 0; j < (width * 3) / 2; j += 3)	{
					buf[0] = image_data[j];
					buf[1] = image_data[j + 1];
					buf[2] = image_data[j + 2];
					image_data[j] = image_data[(width * 3) - j];
					image_data[j + 1] = image_data[(width * 3) - j + 1];
					image_data[j + 2] = image_data[(width * 3) - j + 2];
					image_data[(width * 3) - j] = buf[0];
					image_data[(width * 3) - j + 1] = buf[1];
					image_data[(width * 3) - j + 2] = buf[2];
				}
			}
			else if(rowbytes / width == 4)	{
				unsigned long * image_data = (unsigned long *) rowpointers[i];
				unsigned long buf;
				for(j = 0; j < width / 2; j++)	{
					buf = image_data[j];
					image_data[j] = image_data[width - j];
					image_data[width - j] = buf;
				}
			}
		}
	}

	free(rowpointers);

	if(channels == 4)	{
		type = GL_RGBA;
	}
	else	{
		type = GL_RGB;
	}

	if(rowbytes / width == 1)	{
		type2 = GL_UNSIGNED_BYTE_3_3_2;
	}
	else if(rowbytes / width == 2)	{
		type2 = GL_UNSIGNED_SHORT_5_6_5;
	}
	else if(rowbytes / width == 3)	{
		type2 = GL_UNSIGNED_BYTE;
	}
	else	{
		type2 = GL_UNSIGNED_INT_8_8_8_8;
	}

	glGenTextures(1, num);
	glBindTexture(GL_TEXTURE_2D, *num);
	glTexImage2D(GL_TEXTURE_2D, 0, channels, width, height, 0, type, type2, data);
        glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR);
        glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR);

	free(data);

	return 0;
}
